from brilws.cli.brilcalc_main import brilcalc_main
assert brilcalc_main
from brilws.cli.briltag_main import briltag_main
assert briltag_main
from brilws.cli.brilschema_main import brilschema_main
assert brilschema_main
